@@ -10,7 +10,14 @@ import javax.xml.stream.XMLStreamWriter;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.cfg.CoercionAction;
+import com.fasterxml.jackson.databind.cfg.CoercionInputShape;
+import com.fasterxml.jackson.databind.cfg.MapperBuilder;
+import com.fasterxml.jackson.databind.deser.BeanDeserializerFactory;
+import com.fasterxml.jackson.databind.jsontype.PolymorphicTypeValidator;
+import com.fasterxml.jackson.databind.jsontype.TypeResolverBuilder;
 import com.fasterxml.jackson.dataformat.xml.deser.FromXmlParser;
+import com.fasterxml.jackson.dataformat.xml.deser.XmlDeserializationContext;
 import com.fasterxml.jackson.dataformat.xml.ser.ToXmlGenerator;
 import com.fasterxml.jackson.dataformat.xml.ser.XmlSerializerProvider;
 import com.fasterxml.jackson.dataformat.xml.util.DefaultXmlPrettyPrinter;
@@ -32,6 +39,76 @@ public class XmlMapper extends ObjectMapper
     // as of 2.6
     private static final long serialVersionUID = 1L;
 
+    /**
+     * Builder implementation for constructing {@link XmlMapper} instances.
+     *
+     * @since 2.10
+     */
+    public static class Builder extends MapperBuilder<XmlMapper, Builder>
+    {
+        public Builder(XmlMapper m) {
+            super(m);
+        }
+
+        public Builder enable(FromXmlParser.Feature... features)  {
+            for (FromXmlParser.Feature f : features) {
+                _mapper.enable(f);
+            }
+            return this;
+        }
+
+        public Builder disable(FromXmlParser.Feature... features) {
+            for (FromXmlParser.Feature f : features) {
+                _mapper.disable(f);
+            }
+            return this;
+        }
+
+        public Builder configure(FromXmlParser.Feature feature, boolean state)
+        {
+            if (state) {
+                _mapper.enable(feature);
+            } else {
+                _mapper.disable(feature);
+            }
+            return this;
+        }
+
+        public Builder enable(ToXmlGenerator.Feature... features) {
+            for (ToXmlGenerator.Feature f : features) {
+                _mapper.enable(f);
+            }
+            return this;
+        }
+
+        public Builder disable(ToXmlGenerator.Feature... features) {
+            for (ToXmlGenerator.Feature f : features) {
+                _mapper.disable(f);
+            }
+            return this;
+        }
+
+        public Builder configure(ToXmlGenerator.Feature feature, boolean state)
+        {
+            if (state) {
+                _mapper.enable(feature);
+            } else {
+                _mapper.disable(feature);
+            }
+            return this;
+        }
+        
+        public Builder nameForTextElement(String name) {
+            _mapper.setXMLTextElementName(name);
+            return this;
+        }
+
+        public Builder defaultUseWrapper(boolean state) {
+            _mapper.setDefaultUseWrapper(state);
+            return this;
+        }
+    }
+
     protected final static JacksonXmlModule DEFAULT_XML_MODULE = new JacksonXmlModule();
 
     protected final static DefaultXmlPrettyPrinter DEFAULT_XML_PRETTY_PRINTER = new DefaultXmlPrettyPrinter();
@@ -73,10 +150,10 @@ public class XmlMapper extends ObjectMapper
 
     public XmlMapper(XmlFactory xmlFactory, JacksonXmlModule module)
     {
-        /* Need to override serializer provider (due to root name handling);
-         * deserializer provider fine as is
-         */
-        super(xmlFactory, new XmlSerializerProvider(new XmlRootNameLookup()), null);
+        // Need to override serializer provider (due to root name handling);
+        // deserializer provider fine as is
+        super(xmlFactory, new XmlSerializerProvider(new XmlRootNameLookup()),
+                new XmlDeserializationContext(BeanDeserializerFactory.instance));
         _xmlModule = module;
         // but all the rest is done via Module interface!
         if (module != null) {
@@ -87,6 +164,20 @@ public class XmlMapper extends ObjectMapper
         // 21-Jun-2017, tatu: Seems like there are many cases in XML where ability to coerce empty
         //    String into `null` (where it otherwise is an error) is very useful.
         enable(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT);
+
+        // 13-May-2020, tatu: [dataformat-xml#377] Need to ensure we will keep XML-specific
+        //    Base64 default as "MIME" (not MIME-NO-LINEFEEDS), to preserve pre-2.12
+        //    behavior
+        setBase64Variant(Base64Variants.MIME);
+
+        // 04-Jun-2020, tatu: Use new (2.12) "CoercionConfigs" to support coercion
+        //   from empty and blank Strings to "empty" POJOs etc
+        coercionConfigDefaults()
+            // To allow indentation without problems, need to accept blank String as empty:
+            .setAcceptBlankAsEmpty(Boolean.TRUE)
+            // and then coercion from empty String to empty value, in general
+            .setCoercion(CoercionInputShape.EmptyString, CoercionAction.AsEmpty)
+        ;
     }
 
     /**
@@ -104,11 +195,44 @@ public class XmlMapper extends ObjectMapper
         return new XmlMapper(this);
     }
 
+    /**
+     * @since 2.10
+     */
+    public static XmlMapper.Builder xmlBuilder() {
+        return new XmlMapper.Builder(new XmlMapper());
+    }
+
+    /**
+     * @since 2.10
+     */
+    public static XmlMapper.Builder builder() {
+        return new XmlMapper.Builder(new XmlMapper());
+    }
+
+    /**
+     * @since 2.10
+     */
+    public static XmlMapper.Builder builder(XmlFactory streamFactory) {
+        return new XmlMapper.Builder(new XmlMapper(streamFactory));
+    }
+
     @Override
     public Version version() {
         return PackageVersion.VERSION;
     }
 
+    /*
+    /**********************************************************
+    /* Factory method overrides
+    /**********************************************************
+     */
+
+    @Override // since 2.10
+    protected TypeResolverBuilder<?> _constructDefaultTypeResolverBuilder(DefaultTyping applicability,
+            PolymorphicTypeValidator ptv) {
+        return new DefaultingXmlTypeResolverBuilder(applicability, ptv);
+    }
+
     /*
     /**********************************************************
     /* Additional XML-specific configurations
@@ -122,14 +246,20 @@ public class XmlMapper extends ObjectMapper
      * via {@link JacksonXmlModule}.
      * 
      * @since 2.1
+     *
+     * @deprecated Since 2.10 use {@link Builder#nameForTextElement(String)} instead
      */
+    @Deprecated
     protected void setXMLTextElementName(String name) {
         ((XmlFactory) _jsonFactory).setXMLTextElementName(name);
     }
 
     /**
      * Since 2.7
+     * 
+     * @deprecated Since 2.10 use {@link Builder#defaultUseWrapper(boolean)} instead
      */
+    @Deprecated
     public XmlMapper setDefaultUseWrapper(boolean state) {
         // ser and deser configs should usually have the same introspector, so:
         AnnotationIntrospector ai0 = getDeserializationConfig().getAnnotationIntrospector();
@@ -238,18 +368,4 @@ public class XmlMapper extends ObjectMapper
         // NOTE: above call should do flush(); and we should NOT close here.
         // Finally, 'g' has no buffers to release.
     }
-
-    /*
-    /**********************************************************
-    /* Overridden methods
-    /**********************************************************
-     */
-
-    // 09-May-2016, tatu: Was removed from `jackson-databind` in 2.8; remove from
-    //    here in 2.9.
-    @Deprecated // since 2.6
-//    @Override
-    protected PrettyPrinter _defaultPrettyPrinter() {
-        return new DefaultXmlPrettyPrinter();
-    }
 }
