@@ -8,10 +8,12 @@ import java.lang.annotation.Target;
 /**
  * Annotation that is similar to JAXB <code>javax.xml.bind.annotation.XmlElementWrapper</code>,
  * to indicate wrapper element to use (if any) for Collection types (arrays,
- * <code>java.util.Collection</code>). If defined, a separate container (wrapper) element
+ * {@link java.util.Collection}). If defined, a separate container (wrapper) element
  * is used; if not, entries are written without wrapping.
  */
-@Target({ElementType.FIELD, ElementType.METHOD})
+@Target({ElementType.ANNOTATION_TYPE, ElementType.FIELD, ElementType.METHOD,
+    // @since 2.12 also allowed on (constructor) parameter
+    ElementType.PARAMETER})
 @Retention(RetentionPolicy.RUNTIME)
 public @interface JacksonXmlElementWrapper
 {
