@@ -13,9 +13,13 @@ import javax.xml.stream.XMLStreamWriter;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.base.ParserMinimalBase;
 import com.fasterxml.jackson.core.io.IOContext;
+import com.fasterxml.jackson.core.io.NumberInput;
 import com.fasterxml.jackson.core.util.ByteArrayBuilder;
+import com.fasterxml.jackson.core.util.JacksonFeatureSet;
+
 import com.fasterxml.jackson.dataformat.xml.PackageVersion;
 import com.fasterxml.jackson.dataformat.xml.XmlMapper;
+import com.fasterxml.jackson.dataformat.xml.util.CaseInsensitiveNameSet;
 import com.fasterxml.jackson.dataformat.xml.util.StaxUtil;
 
 /**
@@ -31,6 +35,19 @@ public class FromXmlParser
      */
     public final static String DEFAULT_UNNAMED_TEXT_PROPERTY = "";
 
+    /**
+     * XML format has some peculiarities, indicated via new (2.12) capability
+     * system.
+     *
+     * @since 2.12
+     */
+    protected final static JacksonFeatureSet<StreamReadCapability> XML_READ_CAPABILITIES =
+            DEFAULT_READ_CAPABILITIES
+                .with(StreamReadCapability.DUPLICATE_PROPERTIES)
+                .with(StreamReadCapability.SCALARS_AS_OBJECTS)
+                .with(StreamReadCapability.UNTYPED_SCALARS)
+            ;
+
     /**
      * Enumeration that defines all togglable features for XML parsers.
      */
@@ -38,16 +55,32 @@ public class FromXmlParser
     {
         /**
          * Feature that indicates whether XML Empty elements (ones where there are
-         * no separate start and end tages, but just one tag that ends with "/>")
+         * no separate start and end tags, but just one tag that ends with "/&gt;")
          * are exposed as {@link JsonToken#VALUE_NULL}) or not. If they are not
          * returned as `null` tokens, they will be returned as {@link JsonToken#VALUE_STRING}
          * tokens with textual value of "" (empty String).
          *<p>
-         * Default setting is `true` for backwards compatibility.
+         * Default setting was {@code true} (for backwards compatibility from 2.9 to 2.11 (inclusive)
+         * but was changed in 2.12 to be {@code false} (see [dataformat-xml#411] for details)
          *
          * @since 2.9
          */
-        EMPTY_ELEMENT_AS_NULL(true)
+        EMPTY_ELEMENT_AS_NULL(false),
+
+        // 16-Nov-2020, tatu: would have been nice to add in 2.12 but is not
+        //    trivial to implement... so leaving out for now
+
+        /*
+         * Feature that indicates whether reading operation should check that
+         * the root element's name matches what is expected by read operation:
+         * if enabled and name does not match, an exception will be thrown;
+         * if disabled, no checking is done (any element name will do).
+         *<p>
+         * Default setting is {@code true} for backwards compatibility.
+         *
+         * @since 2.12
+        ENFORCE_VALID_ROOT_NAME(false)
+         */
         ;
 
         final boolean _defaultState;
@@ -85,7 +118,7 @@ public class FromXmlParser
      *<p>
      * Name used for pseudo-property used for returning XML Text value (which does
      * not have actual element name to use). Defaults to empty String, but
-     * may be changed for interoperability reasons: JAXB, for example, uses
+     * may be changed for inter-operability reasons: JAXB, for example, uses
      * "value" as name.
      * 
      * @since 2.1
@@ -146,8 +179,6 @@ public class FromXmlParser
 
     protected String _currText;
 
-    protected Set<String> _namesToWrap;
-
     /*
     /**********************************************************
     /* Parsing state, parsed values
@@ -168,6 +199,27 @@ public class FromXmlParser
      */
     protected byte[] _binaryValue;
 
+    /*
+    /**********************************************************
+    /* Parsing state, number decoding (2.12+)
+    /**********************************************************
+     */
+
+    /**
+     * Bitfield that indicates which numeric representations
+     * have been calculated for the current type
+     */
+    protected int _numTypesValid = NR_UNKNOWN;
+
+    // First primitives
+
+    protected int _numberInt;
+    protected long _numberLong;
+
+    // And then object types
+
+    protected BigInteger _numberBigInt;
+
     /*
     /**********************************************************
     /* Life-cycle
@@ -184,16 +236,39 @@ public class FromXmlParser
         _objectCodec = codec;
         _parsingContext = XmlReadContext.createRootContext(-1, -1);
         _xmlTokens = new XmlTokenStream(xmlReader, ctxt.getSourceReference(),
-                _formatFeatures);
+                    _formatFeatures);
+
+        final int firstToken;
+        try {
+            firstToken = _xmlTokens.initialize();
+        } catch (XMLStreamException e) {
+            StaxUtil.throwAsParseException(e, this);
+            return;
+        }
 
         // 04-Jan-2019, tatu: Root-level nulls need slightly specific handling;
         //    changed in 2.10.2
         if (_xmlTokens.hasXsiNil()) {
             _nextToken = JsonToken.VALUE_NULL;
-        } else if (_xmlTokens.getCurrentToken() == XmlTokenStream.XML_START_ELEMENT) {
-            _nextToken = JsonToken.START_OBJECT;
         } else {
-            _reportError("Internal problem: invalid starting state (%d)", _xmlTokens.getCurrentToken());
+            switch (firstToken) {
+            case XmlTokenStream.XML_START_ELEMENT:
+            case XmlTokenStream.XML_DELAYED_START_ELEMENT:
+                _nextToken = JsonToken.START_OBJECT;
+                break;
+            case XmlTokenStream.XML_ROOT_TEXT:
+                _currText = _xmlTokens.getText();
+                // [dataformat-xml#435]: may get `null` from empty element...
+                // It's complicated.
+                if (_currText == null) {
+                    _nextToken = JsonToken.VALUE_NULL;
+                } else {
+                    _nextToken = JsonToken.VALUE_STRING;
+                }
+                break;
+            default:
+                _reportError("Internal problem: invalid starting state (%s)", _xmlTokens._currentStateDesc());
+            }
         }
     }
 
@@ -218,7 +293,13 @@ public class FromXmlParser
     public void setXMLTextElementName(String name) {
         _cfgNameForTextElement = name;
     }
-    
+
+    /*
+    /**********************************************************************
+    /* Overrides: capability introspection methods
+    /**********************************************************************
+     */
+
     /**
      * XML format does require support from custom {@link ObjectCodec}
      * (that is, {@link XmlMapper}), so need to return true here.
@@ -229,7 +310,18 @@ public class FromXmlParser
     public boolean requiresCustomCodec() {
         return true;
     }
-    
+
+    @Override
+    public boolean canReadObjectId() { return false; }
+
+    @Override
+    public boolean canReadTypeId() { return false; }
+
+    @Override
+    public JacksonFeatureSet<StreamReadCapability> getReadCapabilities() {
+        return XML_READ_CAPABILITIES;
+    }
+
     /*
     /**********************************************************
     /* Extended API, configuration
@@ -275,6 +367,7 @@ public class FromXmlParser
     @Override
     public JsonParser overrideFormatFeatures(int values, int mask) {
         _formatFeatures = (_formatFeatures & ~mask) | (values & mask);
+        _xmlTokens.setFormatFeatures(_formatFeatures);
         return this;
     }
 
@@ -319,21 +412,36 @@ public class FromXmlParser
      * patch versions). So if you have to use it, be prepared for
      * possible additional work.
      * 
-     * @since 2.1
+     * @since 2.12
      */
-    public void addVirtualWrapping(Set<String> namesToWrap)
+    public void addVirtualWrapping(Set<String> namesToWrap0, boolean caseInsensitive)
     {
-        /* 17-Sep-2012, tatu: Not 100% sure why, but this is necessary to avoid
-         *   problems with Lists-in-Lists properties
-         */
-        String name = _xmlTokens.getLocalName();
-        if (name != null && namesToWrap.contains(name)) {
-            _xmlTokens.repeatStartElement();
+//System.out.printf("addVirtualWrapping(%s) at '%s' [case-insensitive? %s]\n", namesToWrap0, _parsingContext.pathAsPointer(), caseInsensitive);
+
+        final Set<String> namesToWrap = caseInsensitive
+                ? CaseInsensitiveNameSet.construct(namesToWrap0)
+                : namesToWrap0;
+
+        // 17-Sep-2012, tatu: Not 100% sure why, but this is necessary to avoid
+        //   problems with Lists-in-Lists properties
+        // 12-May-2020, tatu: But as per [dataformat-xml#86] NOT for root element
+        //   (would still like to know why work-around needed ever, but...)
+        if (!_parsingContext.inRoot()
+                 && !_parsingContext.getParent().inRoot()) {
+            String name = _xmlTokens.getLocalName();
+            if ((name != null) && namesToWrap.contains(name)) {
+//System.out.println("REPEAT from addVirtualWrapping() for '"+name+"'");
+                _xmlTokens.repeatStartElement();
+            }
         }
-        _namesToWrap = namesToWrap;
         _parsingContext.setNamesToWrap(namesToWrap);
     }
 
+    @Deprecated // since 2.12
+    public void addVirtualWrapping(Set<String> namesToWrap) {
+        addVirtualWrapping(namesToWrap, false);
+    }
+
     /*
     /**********************************************************
     /* JsonParser impl
@@ -431,11 +539,11 @@ public class FromXmlParser
     public boolean isExpectedStartArrayToken()
     {
         JsonToken t = _currToken;
-        if (t == JsonToken.START_OBJECT) {        	
+        if (t == JsonToken.START_OBJECT) {
             _currToken = JsonToken.START_ARRAY;
             // Ok: must replace current context with array as well
             _parsingContext.convertToArray();
-//System.out.println(" isExpectedArrayStart: OBJ->Array, wraps now: "+_parsingContext.getNamesToWrap());
+//System.out.println(" FromXmlParser.isExpectedArrayStart(): OBJ->Array");
             // And just in case a field name was to be returned, wipe it
             // 06-Jan-2015, tatu: Actually, could also be empty Object buffered; if so, convert...
             if (_nextToken == JsonToken.END_OBJECT) {
@@ -447,10 +555,72 @@ public class FromXmlParser
             _xmlTokens.skipAttributes();
             return true;
         }
-//System.out.println(" isExpectedArrayStart?: t="+t);
+//System.out.println(" FromXmlParser.isExpectedArrayStart?: t="+t);
         return (t == JsonToken.START_ARRAY);
     }
 
+    /**
+     * Since xml representation can not really distinguish between different
+     * scalar types (numbers, booleans) -- they are all just Character Data,
+     * without schema -- we can try to infer type from intent here.
+     * The main benefit is avoiding checks for coercion.
+     */
+    @Override
+    public boolean isExpectedNumberIntToken()
+    {
+        JsonToken t = _currToken;
+        if (t == JsonToken.VALUE_STRING) {
+            final String text = _currText.trim();
+            final int len = _isIntNumber(text);
+            if (len > 0) {
+                if (len <= 9) {
+                    _numberInt = NumberInput.parseInt(text);
+                    _numTypesValid = NR_INT;
+                    _currToken = JsonToken.VALUE_NUMBER_INT;
+                    return true;
+                }
+                if (len <= 18) { // definitely in long range
+                    long l = NumberInput.parseLong(text);
+                    if (len == 10) {
+                        int asInt = (int) l;
+                        long l2 = (long) asInt;
+                        if (l == l2) {
+                            _numberInt = asInt;
+                            _numTypesValid = NR_INT;
+                            _currToken = JsonToken.VALUE_NUMBER_INT;
+                            return true;
+                        }
+                    }
+                    _numberLong = l;
+                    _numTypesValid = NR_LONG;
+                    _currToken = JsonToken.VALUE_NUMBER_INT;
+                    return true;
+                }
+                // Might still fit within `long`
+                if (len == 19) {
+                    final boolean stillLong;
+                    if (text.charAt(0) == '-') {
+                        stillLong = NumberInput.inLongRange(text.substring(1), true);
+                    } else {
+                        stillLong = NumberInput.inLongRange(text, false);
+                    }
+                    if (stillLong) {
+                        _numberLong = NumberInput.parseLong(text);
+                        _numTypesValid = NR_LONG;
+                        _currToken = JsonToken.VALUE_NUMBER_INT;
+                        return true;
+                    }
+                }
+                // finally, need BigInteger
+                _numberBigInt = new BigInteger(text);
+                _numTypesValid = NR_BIGINT;
+                _currToken = JsonToken.VALUE_NUMBER_INT;
+                return true;
+            }
+        }
+        return (t == JsonToken.VALUE_NUMBER_INT);
+    }
+
     // DEBUGGING
     /*
     @Override
@@ -458,15 +628,16 @@ public class FromXmlParser
     {
         JsonToken t = nextToken0();
         if (t != null) {
+            final String loc = (_parsingContext == null) ? "NULL" : String.valueOf(_parsingContext.pathAsPointer());
             switch (t) {
             case FIELD_NAME:
-                System.out.println("JsonToken: FIELD_NAME '"+_parsingContext.getCurrentName()+"'");
+                System.out.printf("FromXmlParser.nextToken() at '%s': JsonToken.FIELD_NAME '%s'\n", loc, _parsingContext.getCurrentName());
                 break;
             case VALUE_STRING:
-                System.out.println("JsonToken: VALUE_STRING '"+getText()+"'");
+                System.out.printf("FromXmlParser.nextToken() at '%s': JsonToken.VALUE_STRING '%s'\n", loc, getText());
                 break;
             default:
-                System.out.println("JsonToken: "+t);
+                System.out.printf("FromXmlParser.nextToken() at '%s': %s\n", loc, t);
             }
         }
         return t;
@@ -478,10 +649,13 @@ public class FromXmlParser
     public JsonToken nextToken() throws IOException
     {
         _binaryValue = null;
+        _numTypesValid = NR_UNKNOWN;
+//System.out.println("FromXmlParser.nextToken0: _nextToken = "+_nextToken);
         if (_nextToken != null) {
             JsonToken t = _nextToken;
             _currToken = t;
             _nextToken = null;
+
             switch (t) {
             case START_OBJECT:
                 _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
@@ -492,22 +666,19 @@ public class FromXmlParser
             case END_OBJECT:
             case END_ARRAY:
                 _parsingContext = _parsingContext.getParent();
-                _namesToWrap = _parsingContext.getNamesToWrap();
                 break;
             case FIELD_NAME:
                 _parsingContext.setCurrentName(_xmlTokens.getLocalName());
                 break;
             default: // VALUE_STRING, VALUE_NULL
-                // should be fine as is?
+                // 13-May-2020, tatu: [dataformat-xml#397]: advance `index` anyway; not
+                //    used for Object contexts, updated automatically by "createChildXxxContext"
+                _parsingContext.valueStarted();
             }
             return t;
         }
-        int token;
-        try {
-            token = _xmlTokens.next();
-        } catch (XMLStreamException e) {
-            token = StaxUtil.throwAsParseException(e, this);
-        }
+
+        int token = _nextToken();
         // Need to have a loop just because we may have to eat/convert
         // a start-element that indicates an array element.
         while (token == XmlTokenStream.XML_START_ELEMENT) {
@@ -521,11 +692,7 @@ public class FromXmlParser
             if (_parsingContext.inArray()) {
                 // Yup: in array, so this element could be verified; but it won't be
                 // reported anyway, and we need to process following event.
-                try {
-                    token = _xmlTokens.next();
-                } catch (XMLStreamException e) {
-                    StaxUtil.throwAsParseException(e, this);
-                }
+                token = _nextToken();
                 _mayBeLeaf = true;
                 continue;
             }
@@ -534,7 +701,7 @@ public class FromXmlParser
 
             // Ok: virtual wrapping can be done by simply repeating current START_ELEMENT.
             // Couple of ways to do it; but start by making _xmlTokens replay the thing...
-            if (_namesToWrap != null && _namesToWrap.contains(name)) {
+            if (_parsingContext.shouldWrap(name)) {
                 _xmlTokens.repeatStartElement();
             }
 
@@ -560,12 +727,13 @@ public class FromXmlParser
                     }
                     // 07-Sep-2019, tatu: for [dataformat-xml#353], must NOT return second null
                     if (_currToken != JsonToken.VALUE_NULL) {
+                        // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
+                        _parsingContext.valueStarted();
                         return (_currToken = JsonToken.VALUE_NULL);
                     }
                 }
                 _currToken = _parsingContext.inArray() ? JsonToken.END_ARRAY : JsonToken.END_OBJECT;
                 _parsingContext = _parsingContext.getParent();
-                _namesToWrap = _parsingContext.getNamesToWrap();
                 return _currToken;
 
             case XmlTokenStream.XML_ATTRIBUTE_NAME:
@@ -581,6 +749,8 @@ public class FromXmlParser
                 return (_currToken = JsonToken.FIELD_NAME);
             case XmlTokenStream.XML_ATTRIBUTE_VALUE:
                 _currText = _xmlTokens.getText();
+                // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
+                _parsingContext.valueStarted();
                 return (_currToken = JsonToken.VALUE_STRING);
             case XmlTokenStream.XML_TEXT:
                 _currText = _xmlTokens.getText();
@@ -590,36 +760,50 @@ public class FromXmlParser
                     // we had an empty String (or all white space), and we are
                     // deserializing an array, we better hide the empty text.
                     // Also: must skip following END_ELEMENT
-                    try {
-                        _xmlTokens.skipEndElement();
-                    } catch (XMLStreamException e) {
-                        StaxUtil.throwAsParseException(e, this);
-                    }
-                    if (_parsingContext.inArray()) {
-                        if (_isEmpty(_currText)) {
-                            // 06-Jan-2015, tatu: as per [dataformat-xml#180], need to
-                            //    expose as empty Object, not null (or, worse, as used to
-                            //    be done, by swallowing the token)
-                            _nextToken = JsonToken.END_OBJECT;
-                            _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
-                            return (_currToken = JsonToken.START_OBJECT);
+                    // 05-Jun-2020, tatu: ... if there is one; we may actually alternatively
+                    //   get START_ELEMENT for "mixed content" case; if so, need to change to
+                    //   expose "XmlText" as separate property
+                    token = _nextToken();
+
+                    if (token == XmlTokenStream.XML_END_ELEMENT) {
+                        if (_parsingContext.inArray()) {
+                            if (XmlTokenStream._allWs(_currText)) {
+                                // 06-Jan-2015, tatu: as per [dataformat-xml#180], need to
+                                //    expose as empty Object, not null (or, worse, as used to
+                                //    be done, by swallowing the token)
+                                _nextToken = JsonToken.END_OBJECT;
+                                _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
+                                return (_currToken = JsonToken.START_OBJECT);
+                            }
                         }
+                        return (_currToken = JsonToken.VALUE_STRING);
                     }
-                    return (_currToken = JsonToken.VALUE_STRING);
+                    if (token != XmlTokenStream.XML_START_ELEMENT) {
+                        throw new JsonParseException(this, String.format(
+"Internal error: Expected END_ELEMENT (%d) or START_ELEMENT (%d), got event of type %d",
+XmlTokenStream.XML_END_ELEMENT, XmlTokenStream.XML_START_ELEMENT, token));
+                    }
+                    // fall-through, except must create new context AND push back
+                    // START_ELEMENT we just saw:
+                    _xmlTokens.pushbackCurrentToken();
+                    _parsingContext = _parsingContext.createChildObjectContext(-1, -1);
                 }
                 // [dataformat-xml#177]: empty text may also need to be skipped
                 // but... [dataformat-xml#191]: looks like we can't short-cut, must
                 // loop over again
                 if (_parsingContext.inObject()) {
-                    if ((_currToken != JsonToken.FIELD_NAME) && _isEmpty(_currText)) {
-                        try {
-                            token = _xmlTokens.next();
-                        } catch (XMLStreamException e) {
-                            StaxUtil.throwAsParseException(e, this);
-                        }
+                    if ((_currToken != JsonToken.FIELD_NAME) && XmlTokenStream._allWs(_currText)) {
+                        token = _nextToken();
+                        continue;
+                    }
+                } else if (_parsingContext.inArray()) {
+                    // [dataformat-xml#319] Aaaaand for Arrays too
+                    if (XmlTokenStream._allWs(_currText)) {
+                        token = _nextToken();
                         continue;
                     }
                 }
+
                 // If not a leaf (or otherwise ignorable), need to transform into property...
                 _parsingContext.setCurrentName(_cfgNameForTextElement);
                 _nextToken = JsonToken.VALUE_STRING;
@@ -663,19 +847,15 @@ public class FromXmlParser
 
             // expected case; yes, got a String
             if (t == JsonToken.VALUE_STRING) {
+                // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
+                _parsingContext.valueStarted();
                 return _currText;
             }
             _updateState(t);
             return null;
         }
 
-        int token;
-
-        try {
-            token = _xmlTokens.next();
-        } catch (XMLStreamException e) {
-            token = StaxUtil.throwAsParseException(e, this);
-        }
+        int token = _nextToken();
 
         // mostly copied from 'nextToken()'
         while (token == XmlTokenStream.XML_START_ELEMENT) {
@@ -686,17 +866,14 @@ public class FromXmlParser
                 return null;
             }
             if (_parsingContext.inArray()) {
-                try {
-                    token = _xmlTokens.next();
-                } catch (XMLStreamException e) {
-                    StaxUtil.throwAsParseException(e, this);
-                }
+                token = _nextToken();
                 _mayBeLeaf = true;
                 continue;
             }
             String name = _xmlTokens.getLocalName();
             _parsingContext.setCurrentName(name);
-            if (_namesToWrap != null && _namesToWrap.contains(name)) {
+            if (_parsingContext.shouldWrap(name)) {
+//System.out.println("REPEAT from nextTextValue()");
                 _xmlTokens.repeatStartElement();
             }
             _mayBeLeaf = true;
@@ -711,11 +888,12 @@ public class FromXmlParser
                 // NOTE: this is different from nextToken() -- produce "", NOT null
                 _mayBeLeaf = false;
                 _currToken = JsonToken.VALUE_STRING;
+                // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
+                _parsingContext.valueStarted();
                 return (_currText = "");
             }
             _currToken = _parsingContext.inArray() ? JsonToken.END_ARRAY : JsonToken.END_OBJECT;
             _parsingContext = _parsingContext.getParent();
-            _namesToWrap = _parsingContext.getNamesToWrap();
             break;
         case XmlTokenStream.XML_ATTRIBUTE_NAME:
             // If there was a chance of leaf node, no more...
@@ -732,19 +910,19 @@ public class FromXmlParser
             break;
         case XmlTokenStream.XML_ATTRIBUTE_VALUE:
             _currToken = JsonToken.VALUE_STRING;
+            // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
+            _parsingContext.valueStarted();
             return (_currText = _xmlTokens.getText());
         case XmlTokenStream.XML_TEXT:
             _currText = _xmlTokens.getText();
             if (_mayBeLeaf) {
                 _mayBeLeaf = false;
                 // Also: must skip following END_ELEMENT
-                try {
-                    _xmlTokens.skipEndElement();
-                } catch (XMLStreamException e) {
-                    StaxUtil.throwAsParseException(e, this);
-                }
+                _skipEndElement();
                 // NOTE: this is different from nextToken() -- NO work-around
                 // for otherwise empty List/array
+                // 13-May-2020, tatu: [dataformat-xml#397]: advance `index`
+                _parsingContext.valueStarted();
                 _currToken = JsonToken.VALUE_STRING;
                 return _currText;
             }
@@ -774,7 +952,6 @@ public class FromXmlParser
         case END_OBJECT:
         case END_ARRAY:
             _parsingContext = _parsingContext.getParent();
-            _namesToWrap = _parsingContext.getNamesToWrap();
             break;
         case FIELD_NAME:
             _parsingContext.setCurrentName(_xmlTokens.getLocalName());
@@ -806,58 +983,6 @@ public class FromXmlParser
         }
     }
 
-    // @since 2.1
-    @Override
-    public final String getValueAsString() throws IOException {
-        return getValueAsString(null);
-    }
-
-    @Override
-    public String getValueAsString(String defValue) throws IOException
-    {
-        JsonToken t = _currToken;
-        if (t == null) {
-            return null;
-        }
-        switch (t) {
-        case FIELD_NAME:
-            return getCurrentName();
-        case VALUE_STRING:
-            return _currText;
-        case START_OBJECT:
-            // the interesting case; may be able to convert certain kinds of
-            // elements (specifically, ones with attributes, CDATA only content)
-            // into VALUE_STRING
-            try {
-                String str = _xmlTokens.convertToString();
-                if (str != null) {
-                    // need to convert token, as well as "undo" START_OBJECT
-                    // note: Should NOT update context, because we will still be getting
-                    // matching END_OBJECT, which will undo contexts properly
-                    _parsingContext = _parsingContext.getParent();
-                    _namesToWrap = _parsingContext.getNamesToWrap();
-                    _currToken = JsonToken.VALUE_STRING;
-                    _nextToken = null;
-                    // One more thing: must explicitly skip the END_OBJECT that would follow
-                    try {
-                        _xmlTokens.skipEndElement();
-                    } catch (XMLStreamException e) {
-                        StaxUtil.throwAsParseException(e, this);
-                    }
-                    return (_currText = str);
-                }
-            } catch (XMLStreamException e) {
-                StaxUtil.throwAsParseException(e, this);
-            }
-            return null;
-        default:
-            if (_currToken.isScalarValue()) {
-                return _currToken.asString();
-            }
-        }
-        return defValue;
-    }
-    
     @Override
     public char[] getTextCharacters() throws IOException {
         String text = getText();
@@ -895,7 +1020,7 @@ public class FromXmlParser
         writer.write(str);
         return str.length();
     }
-    
+
     /*
     /**********************************************************
     /* Public API, access to token information, binary
@@ -936,58 +1061,246 @@ public class FromXmlParser
         _decodeBase64(str, builder, b64variant);
         return builder.toByteArray();
     }
-    
+
     /*
     /**********************************************************
-    /* Numeric accessors
+    /* Numeric accessors (implemented since 2.12)
     /**********************************************************
      */
 
     @Override
-    public BigInteger getBigIntegerValue() throws IOException {
-        // TODO Auto-generated method stub
-        return null;
-    }
-
-    @Override
-    public BigDecimal getDecimalValue() throws IOException {
-        // TODO Auto-generated method stub
-        return null;
-    }
-
-    @Override
-    public double getDoubleValue() throws IOException {
-        // TODO Auto-generated method stub
-        return 0;
-    }
-
-    @Override
-    public float getFloatValue() throws IOException {
-        // TODO Auto-generated method stub
-        return 0;
-    }
-
-    @Override
-    public int getIntValue() throws IOException {
-        // TODO Auto-generated method stub
-        return 0;
-    }
-
-    @Override
-    public long getLongValue() throws IOException {
-        // TODO Auto-generated method stub
-        return 0;
+    public boolean isNaN() {
+        return false; // can't have since we only coerce integers
     }
 
     @Override
     public NumberType getNumberType() throws IOException {
-        // TODO Auto-generated method stub
-        return null;
+        if (_numTypesValid == NR_UNKNOWN) {
+            _checkNumericValue(NR_UNKNOWN); // will also check event type
+        }
+        // Only integer types supported so...
+        
+        if ((_numTypesValid & NR_INT) != 0) {
+            return NumberType.INT;
+        }
+        if ((_numTypesValid & NR_LONG) != 0) {
+            return NumberType.LONG;
+        }
+        return NumberType.BIG_INTEGER;
     }
 
     @Override
     public Number getNumberValue() throws IOException {
-        // TODO Auto-generated method stub
+        if (_numTypesValid == NR_UNKNOWN) {
+            _checkNumericValue(NR_UNKNOWN); // will also check event type
+        }
+        // Only integer types supported so...
+
+        if ((_numTypesValid & NR_INT) != 0) {
+            return _numberInt;
+        }
+        if ((_numTypesValid & NR_LONG) != 0) {
+            return _numberLong;
+        }
+        if ((_numTypesValid & NR_BIGINT) != 0) {
+            return _numberBigInt;
+        }
+        _throwInternal();
+        return null;
+    }
+
+    @Override
+    public int getIntValue() throws IOException {
+        if ((_numTypesValid & NR_INT) == 0) {
+            if (_numTypesValid == NR_UNKNOWN) { // not parsed at all
+                _checkNumericValue(NR_INT); // will also check event type
+            }
+            if ((_numTypesValid & NR_INT) == 0) { // wasn't an int natively?
+                _convertNumberToInt(); // let's make it so, if possible
+            }
+        }
+        return _numberInt;
+    }
+
+    @Override
+    public long getLongValue() throws IOException {
+        if ((_numTypesValid & NR_LONG) == 0) {
+            if (_numTypesValid == NR_UNKNOWN) {
+                _checkNumericValue(NR_LONG);
+            }
+            if ((_numTypesValid & NR_LONG) == 0) {
+                _convertNumberToLong();
+            }
+        }
+        return _numberLong;
+    }
+
+    @Override
+    public BigInteger getBigIntegerValue() throws IOException {
+        if ((_numTypesValid & NR_BIGINT) == 0) {
+            if (_numTypesValid == NR_UNKNOWN) {
+                _checkNumericValue(NR_BIGINT);
+            }
+            if ((_numTypesValid & NR_BIGINT) == 0) {
+                _convertNumberToBigInteger();
+            }
+        }
+        return _numberBigInt;
+    }
+
+    @Override
+    public float getFloatValue() throws IOException {
+        if ((_numTypesValid & NR_FLOAT) == 0) {
+            if (_numTypesValid == NR_UNKNOWN) {
+                _checkNumericValue(NR_FLOAT);
+            }
+        }
+        return _convertNumberToFloat();
+    }
+
+    @Override
+    public double getDoubleValue() throws IOException {
+        if ((_numTypesValid & NR_DOUBLE) == 0) {
+            if (_numTypesValid == NR_UNKNOWN) {
+                _checkNumericValue(NR_DOUBLE);
+            }
+        }
+        return _convertNumberToDouble();
+    }
+
+    @Override
+    public BigDecimal getDecimalValue() throws IOException {
+        if ((_numTypesValid & NR_BIGDECIMAL) == 0) {
+            if (_numTypesValid == NR_UNKNOWN) {
+                _checkNumericValue(NR_BIGDECIMAL);
+            }
+        }
+        return _convertNumberToBigDecimal();
+    }
+
+    // // // Helper methods for Numeric accessors
+
+    protected final void _checkNumericValue(int expType) throws IOException {
+        if (_currToken == JsonToken.VALUE_NUMBER_INT) {
+            return;
+        }
+        _reportError("Current token ("+currentToken()+") not numeric, can not use numeric value accessors");
+    }
+
+    // NOTE: copied from `StdDeserializer`...
+    protected final int _isIntNumber(String text)
+    {
+        final int len = text.length();
+        if (len > 0) {
+            char c = text.charAt(0);
+            // skip leading negative sign, do NOT allow leading plus
+            final int start = (c == '-') ? 1 : 0;
+            for (int i = start; i < len; ++i) {
+                int ch = text.charAt(i);
+                if (ch > '9' || ch < '0') {
+                    return -1;
+                }
+            }
+            return len - start;
+        }
+        return 0;
+    }
+
+    protected void _convertNumberToInt() throws IOException
+    {
+        // First, converting from long ought to be easy
+        if ((_numTypesValid & NR_LONG) != 0) {
+            // Let's verify it's lossless conversion by simple roundtrip
+            int result = (int) _numberLong;
+            if (((long) result) != _numberLong) {
+                _reportError("Numeric value ("+getText()+") out of range of int");
+            }
+            _numberInt = result;
+        } else if ((_numTypesValid & NR_BIGINT) != 0) {
+            if (BI_MIN_INT.compareTo(_numberBigInt) > 0 
+                    || BI_MAX_INT.compareTo(_numberBigInt) < 0) {
+                reportOverflowInt();
+            }
+            _numberInt = _numberBigInt.intValue();
+        } else {
+            _throwInternal();
+        }
+        _numTypesValid |= NR_INT;
+    }
+    
+    protected void _convertNumberToLong() throws IOException
+    {
+        if ((_numTypesValid & NR_INT) != 0) {
+            _numberLong = (long) _numberInt;
+        } else if ((_numTypesValid & NR_BIGINT) != 0) {
+            if (BI_MIN_LONG.compareTo(_numberBigInt) > 0 
+                    || BI_MAX_LONG.compareTo(_numberBigInt) < 0) {
+                reportOverflowLong();
+            }
+            _numberLong = _numberBigInt.longValue();
+        } else {
+            _throwInternal();
+        }
+        _numTypesValid |= NR_LONG;
+    }
+    
+    protected void _convertNumberToBigInteger() throws IOException
+    {
+        if ((_numTypesValid & NR_LONG) != 0) {
+            _numberBigInt = BigInteger.valueOf(_numberLong);
+        } else if ((_numTypesValid & NR_INT) != 0) {
+            _numberBigInt = BigInteger.valueOf(_numberInt);
+        } else {
+            _throwInternal();
+        }
+        _numTypesValid |= NR_BIGINT;
+    }
+
+    protected float _convertNumberToFloat() throws IOException
+    {
+        // Note: this MUST start with more accurate representations, since we don't know which
+        //  value is the original one (others get generated when requested)
+        if ((_numTypesValid & NR_BIGINT) != 0) {
+            return _numberBigInt.floatValue();
+        }
+        if ((_numTypesValid & NR_LONG) != 0) {
+            return (float) _numberLong;
+        }
+        if ((_numTypesValid & NR_INT) != 0) {
+            return (float) _numberInt;
+        }
+        _throwInternal();
+        return 0.0f;
+    }
+    
+    protected double _convertNumberToDouble() throws IOException
+    {
+        // same as above, start from more to less accurate
+        if ((_numTypesValid & NR_BIGINT) != 0) {
+            return _numberBigInt.doubleValue();
+        }
+        if ((_numTypesValid & NR_LONG) != 0) {
+            return (double) _numberLong;
+        }
+        if ((_numTypesValid & NR_INT) != 0) {
+            return (double) _numberInt;
+        }
+        _throwInternal();
+        return 0.0;
+    }
+
+    protected BigDecimal _convertNumberToBigDecimal() throws IOException
+    {
+        if ((_numTypesValid & NR_BIGINT) != 0) {
+            return new BigDecimal(_numberBigInt);
+        }
+        if ((_numTypesValid & NR_LONG) != 0) {
+            return BigDecimal.valueOf(_numberLong);
+        }
+        if ((_numTypesValid & NR_INT) != 0) {
+            return BigDecimal.valueOf(_numberInt);
+        }
+        _throwInternal();
         return null;
     }
 
@@ -1039,20 +1352,25 @@ public class FromXmlParser
         return _byteArrayBuilder;
     }
 
-    protected boolean _isEmpty(String str)
-    {
-        int len = (str == null) ? 0 : str.length();
-        if (len > 0) {
-            for (int i = 0; i < len; ++i) {
-                if (str.charAt(i) > ' ') {
-                    return false;
-                }
-            }
-        }
-        return true;
-    }
-
-    private <T> T  _internalErrorUnknownToken(Object token) {
+    private <T> T _internalErrorUnknownToken(Object token) {
         throw new IllegalStateException("Internal error: unrecognized XmlTokenStream token: "+token);
     }
+
+    protected int _nextToken() throws IOException {
+        try {
+            return _xmlTokens.next();
+        } catch (XMLStreamException e) {
+            return StaxUtil.throwAsParseException(e, this);
+        }
+    }
+
+    protected void _skipEndElement() throws IOException {
+        try {
+            _xmlTokens.skipEndElement();
+        } catch (XMLStreamException e) {
+            StaxUtil.throwAsParseException(e, this);
+        } catch (Exception e) {
+            throw new JsonParseException(this, e.getMessage(), e);
+        }
+    }
 }
