@@ -105,6 +105,7 @@ public final class XmlReadContext
     
     public final XmlReadContext createChildArrayContext(int lineNr, int colNr)
     {
+        ++_index; // not needed for Object, but does not hurt so no need to check curr type
         XmlReadContext ctxt = _child;
         if (ctxt == null) {
             _child = ctxt = new XmlReadContext(this, TYPE_ARRAY, lineNr, colNr);
@@ -116,6 +117,7 @@ public final class XmlReadContext
 
     public final XmlReadContext createChildObjectContext(int lineNr, int colNr)
     {
+        ++_index; // not needed for Object, but does not hurt so no need to check curr type
         XmlReadContext ctxt = _child;
         if (ctxt == null) {
             _child = ctxt = new XmlReadContext(this, TYPE_OBJECT, lineNr, colNr);
@@ -127,7 +129,7 @@ public final class XmlReadContext
 
     /*
     /**********************************************************
-    /* Abstract method implementation
+    /* Abstract method implementation, overrides
     /**********************************************************
      */
 
@@ -140,26 +142,6 @@ public final class XmlReadContext
     @Override
     public final XmlReadContext getParent() { return _parent; }
 
-    /*
-    /**********************************************************
-    /* State changes
-    /**********************************************************
-     */
-
-    public final boolean expectComma() {
-        throw new UnsupportedOperationException();
-    }
-
-    public void setCurrentName(String name) {
-        _currentName = name;
-    }
-
-    /*
-    /**********************************************************
-    /* Extended API
-    /**********************************************************
-     */
-
     /**
      * @return Location pointing to the point where the context
      *   start marker was found
@@ -172,13 +154,28 @@ public final class XmlReadContext
         return new JsonLocation(srcRef, totalChars, _lineNr, _columnNr);
     }
 
-    public void setNamesToWrap(Set<String> namesToWrap) {
-        _namesToWrap = namesToWrap;
+    /*
+    /**********************************************************
+    /* Extended API
+    /**********************************************************
+     */
+
+    /**
+     * Method called to mark start of new value, mostly to update `index`
+     * for Array and Root contexts.
+     *
+     * @since 2.12
+     */
+    public final void valueStarted() {
+        ++_index;
     }
 
-    @Deprecated // since 2.11.1
-    public Set<String> getNamesToWrap() {
-        return _namesToWrap;
+    public void setCurrentName(String name) {
+        _currentName = name;
+    }
+
+    public void setNamesToWrap(Set<String> namesToWrap) {
+        _namesToWrap = namesToWrap;
     }
 
     // @since 2.11.1
