@@ -25,14 +25,22 @@ import com.fasterxml.jackson.core.JsonLocation;
 public class XmlTokenStream
 {
     // // // main token states:
-    
+
     public final static int XML_START_ELEMENT = 1;
     public final static int XML_END_ELEMENT = 2;
     public final static int XML_ATTRIBUTE_NAME = 3;
     public final static int XML_ATTRIBUTE_VALUE = 4;
     public final static int XML_TEXT = 5;
 
-    public final static int XML_END = 6;
+    // New in 2.12: needed to "re-process" previously encountered START_ELEMENT,
+    // with possible leading text
+    public final static int XML_DELAYED_START_ELEMENT = 6;
+
+    // 2.12 also exposes "root scalars" as-is, instead of wrapping as Objects; this
+    // needs some more state management too
+    public final static int XML_ROOT_TEXT = 7;
+
+    public final static int XML_END = 8;
 
     // // // token replay states
 
@@ -79,11 +87,11 @@ public class XmlTokenStream
     protected boolean _xsiNilFound;
 
     /**
-     * If true we have a START_ELEMENT with mixed text
+     * Flag set true if current event is {@code XML_TEXT} and there is START_ELEMENT
      *
-     * @since 2.8
+     * @since 2.12
      */
-    protected boolean _mixedText;
+    protected boolean _startElementAfterText;
 
     /**
      * Index of the next attribute of the current START_ELEMENT
@@ -96,8 +104,20 @@ public class XmlTokenStream
 
     protected String _namespaceURI;
 
+    /**
+     * Current text value for TEXT_VALUE returned
+     */
     protected String _textValue;
-    
+
+    /**
+     * Marker flag set if caller wants to "push back" current token so
+     * that next call to {@link #next()} should simply be given what was
+     * already read.
+     *
+     * @since 2.12
+     */
+    protected boolean _repeatCurrentToken;
+
     /*
     /**********************************************************************
     /* State for handling virtual wrapping
@@ -132,18 +152,57 @@ public class XmlTokenStream
             int formatFeatures)
     {
         _sourceReference = sourceRef;
-        // Let's ensure we point to START_ELEMENT...
-        if (xmlReader.getEventType() != XMLStreamConstants.START_ELEMENT) {
-            throw new IllegalArgumentException("Invalid XMLStreamReader passed: should be pointing to START_ELEMENT ("
-                    +XMLStreamConstants.START_ELEMENT+"), instead got "+xmlReader.getEventType());
-        }
+        _formatFeatures = formatFeatures;
         _xmlReader = Stax2ReaderAdapter.wrapIfNecessary(xmlReader);
+    }
+
+    /**
+     * Second part of initialization, to be called immediately after construction
+     *
+     * @since 2.12
+     */
+    public int initialize() throws XMLStreamException
+    {
+        // Let's ensure we point to START_ELEMENT...
+        if (_xmlReader.getEventType() != XMLStreamConstants.START_ELEMENT) {
+            throw new IllegalArgumentException("Invalid XMLStreamReader passed: should be pointing to START_ELEMENT ("
+                    +XMLStreamConstants.START_ELEMENT+"), instead got "+_xmlReader.getEventType());
+        }
         _localName = _xmlReader.getLocalName();
         _namespaceURI = _xmlReader.getNamespaceURI();
-        _formatFeatures = formatFeatures;
 
         _checkXsiAttributes(); // sets _attributeCount, _nextAttributeIndex
-        _currentState = XML_START_ELEMENT;
+
+        // 02-Jul-2020, tatu: Two choices: if child elements OR attributes, expose
+        //    as Object value; otherwise expose as Text
+        if (_xsiNilFound || _attributeCount > 0) {
+            return (_currentState = XML_START_ELEMENT);
+        }
+
+        // copied from START_ELEMENT section of _next():
+        final String text = _collectUntilTag();
+        if (text == null) {
+            // 30-Nov-2020, tatu: [dataformat-xml#435], this is tricky
+            //   situation since we got coerced `null`... but at least for
+            //   now will have to report as "root String" (... with null contents)
+            _textValue = null;
+            _startElementAfterText = false;
+            return (_currentState = XML_ROOT_TEXT);
+        }
+
+        final boolean startElementNext = _xmlReader.getEventType() == XMLStreamReader.START_ELEMENT;
+        // If we have no/all-whitespace text followed by START_ELEMENT, ignore text
+        if (startElementNext) {
+            if (_allWs(text)) {
+                _textValue = null;
+                return (_currentState = XML_DELAYED_START_ELEMENT);
+            }
+            _textValue = text;
+            return (_currentState = XML_DELAYED_START_ELEMENT);
+        }
+        _startElementAfterText = false;
+        _textValue = text;
+        return (_currentState = XML_ROOT_TEXT);
     }
 
     public XMLStreamReader2 getXmlReader() {
@@ -172,8 +231,12 @@ public class XmlTokenStream
         case XML_START_ELEMENT: 
             System.out.printf(" XmlTokenStream.next(): XML_START_ELEMENT '%s' %s\n", _localName, _loc());
             break;
+        case XML_DELAYED_START_ELEMENT: 
+            System.out.printf(" XmlTokenStream.next(): XML_DELAYED_START_ELEMENT '%s' %s\n", _localName, _loc());
+            break;
         case XML_END_ELEMENT: 
-            System.out.printf(" XmlTokenStream.next(): XML_END_ELEMENT '%s' %s\n", _localName, _loc());
+            // 24-May-2020, tatu: no name available for end element so do not print
+            System.out.printf(" XmlTokenStream.next(): XML_END_ELEMENT %s\n", _loc());
             break;
         case XML_ATTRIBUTE_NAME: 
             System.out.printf(" XmlTokenStream.next(): XML_ATTRIBUTE_NAME '%s' %s\n", _localName, _loc());
@@ -202,6 +265,10 @@ public class XmlTokenStream
 //    public int next0() throws XMLStreamException
     public int next() throws XMLStreamException
     {
+        if (_repeatCurrentToken) {
+            _repeatCurrentToken = false;
+            return _currentState;
+        }
         if (_repeatElement != 0) {
             return (_currentState = _handleRepeatElement());
         }
@@ -212,14 +279,24 @@ public class XmlTokenStream
     {
         int type = next();
         if (type != XML_END_ELEMENT) {
-            throw new IOException("Expected END_ELEMENT, got event of type "+type);
+            throw new IOException(String.format(
+                    "Internal error: Expected END_ELEMENT, got event of type %s",
+                    _stateDesc(type)));
         }
     }
 
     public int getCurrentToken() { return _currentState; }
 
     public String getText() { return _textValue; }
+
+    /**
+     * Accessor for local name of current named event (that is,
+     * {@code XML_START_ELEMENT} or {@code XML_ATTRIBUTE_NAME}).
+     *<p>
+     * NOTE: name NOT accessible on {@code XML_END_ELEMENT}
+     */
     public String getLocalName() { return _localName; }
+
     public String getNamespaceURI() { return _namespaceURI; }
 
     public boolean hasXsiNil() {
@@ -260,7 +337,7 @@ public class XmlTokenStream
      */
     protected void repeatStartElement()
     {
-//System.out.println(" -> repeatStartElement for "+_localName+", _currentWrapper was: "+_currentWrapper);
+//System.out.println(" XmlTokenStream.repeatStartElement() for <"+_localName+">, _currentWrapper was: "+_currentWrapper);
         // sanity check: can only be used when just returned START_ELEMENT:
         if (_currentState != XML_START_ELEMENT) {
             // 14-May-2020, tatu: Looks like we DO end up here with empty Lists; if so,
@@ -268,8 +345,7 @@ public class XmlTokenStream
             if (_currentState == XML_END_ELEMENT) {
                 return;
             }
-            throw new IllegalStateException("Current state not XML_START_ELEMENT ("
-                    +XML_START_ELEMENT+") but "+_currentState);
+            throw new IllegalStateException("Current state not XML_START_ELEMENT but "+_currentStateDesc());
         }
         // Important: add wrapper, to keep track...
         if (_currentWrapper == null) {
@@ -281,6 +357,17 @@ public class XmlTokenStream
         _repeatElement = REPLAY_START_DUP;
     }
 
+    /**
+     * Method that can be called to ask stream to literally just return current token
+     * with the next call to {@link #next()}, without more work.
+     *
+     * @since 2.12
+     */
+    protected void pushbackCurrentToken()
+    {
+        _repeatCurrentToken = true;
+    }
+
     /**
      * Method called to skip any attributes current START_ELEMENT may have,
      * so that they are not returned as token.
@@ -289,55 +376,29 @@ public class XmlTokenStream
      */
     protected void skipAttributes()
     {
-        if (_currentState == XML_ATTRIBUTE_NAME) {
+//System.out.println(" XmlTokenStream.skipAttributes(), state: "+_currentStateDesc());
+        switch (_currentState) {
+        case XML_ATTRIBUTE_NAME:
             _attributeCount = 0;
             _currentState = XML_START_ELEMENT;
-        } else if (_currentState == XML_START_ELEMENT) {
-            /* 06-Jan-2012, tatu: As per [#47] it looks like we should NOT do anything
-             *   in this particular case, because it occurs when original element had
-             *   no attributes and we now point to the first child element.
-             */
+            break;
+        case XML_START_ELEMENT:
+            // 06-Jan-2012, tatu: As per [#47] it looks like we should NOT do anything
+            //   in this particular case, because it occurs when original element had
+            //   no attributes and we now point to the first child element.
 //              _attributeCount = 0;
-        } else if (_currentState == XML_TEXT) {
-            ; // nothing to do... is it even legal?
-        } else {
-            throw new IllegalStateException("Current state not XML_START_ELEMENT or XML_ATTRIBUTE_NAME ("
-                    +XML_START_ELEMENT+") but "+_currentState);
+            break;
+        case XML_TEXT:
+            break; // nothing to do... is it even legal?
+        case XML_DELAYED_START_ELEMENT:
+            // 03-Jul-2020, tatu: and here nothing to do either... ?
+            break;
+        default:
+            throw new IllegalStateException(
+"Current state not XML_START_ELEMENT or XML_ATTRIBUTE_NAME but "+_currentStateDesc());
         }
     }
 
-    /**
-     * Helper method called by XML String deserializer to concatenate textual contents
-     * contained in logical "Object": mostly just to skip attribute values.
-     */
-    protected String convertToString() throws XMLStreamException
-    {
-        // only applicable to cases where START_OBJECT was induced by attributes
-        if (_currentState != XML_ATTRIBUTE_NAME || _nextAttributeIndex != 0) {
-            return null;
-        }
-        String text = _collectUntilTag();
-        // 23-Dec-2015, tatu: Used to require text not to be null, but as per
-        //   [dataformat-xml#167], empty tag does count
-        if (_xmlReader.getEventType() == XMLStreamReader.END_ELEMENT) {
-            if (text == null) {
-                text = "";
-            }
-            if (_currentWrapper != null) {
-                _currentWrapper = _currentWrapper.getParent();
-            }
-            // just for diagnostics, reset to element name (from first attribute name)
-            _localName = _xmlReader.getLocalName();
-            _namespaceURI = _xmlReader.getNamespaceURI();
-            _attributeCount = 0;
-            _currentState = XML_TEXT;
-            _textValue = text;
-            return text;
-        }
-        // Anything to do in failed case? Roll back whatever we found or.. ?
-        return null;
-    }
-
     /*
     /**********************************************************************
     /* Internal methods, parsing
@@ -346,6 +407,7 @@ public class XmlTokenStream
 
     private final int _next() throws XMLStreamException
     {
+//System.out.println(" XmlTokenStream._next(), state: "+_currentStateDesc());
         switch (_currentState) {
         case XML_ATTRIBUTE_VALUE:
             ++_nextAttributeIndex;
@@ -365,6 +427,7 @@ public class XmlTokenStream
                 throw new IllegalStateException("Unexpected START_ELEMENT after null token");
             }
             if (_nextAttributeIndex < _attributeCount) {
+//System.out.println(" XmlTokenStream._next(): Got attr(s)!");
                 _localName = _xmlReader.getAttributeLocalName(_nextAttributeIndex);
                 _namespaceURI = _xmlReader.getAttributeNamespace(_nextAttributeIndex);
                 _textValue = _xmlReader.getAttributeValue(_nextAttributeIndex);
@@ -372,53 +435,85 @@ public class XmlTokenStream
             }
             // otherwise need to find START/END_ELEMENT or text
             String text = _collectUntilTag();
+//System.out.println(" XmlTokenStream._next(): _collectUntilTag -> '"+text+"'");
             final boolean startElementNext = _xmlReader.getEventType() == XMLStreamReader.START_ELEMENT;
+//System.out.println(" XmlTokenStream._next(): startElementNext? "+startElementNext);
             // If we have no/all-whitespace text followed by START_ELEMENT, ignore text
             if (startElementNext) {
-                if (text == null || _allWs(text)) {
-                    _mixedText = false;
+                if (_allWs(text)) {
+                    _startElementAfterText = false;
                     return _initStartElement();
                 }
-                _mixedText = true;
+                _startElementAfterText = true;
                 _textValue = text;
+                
                 return (_currentState = XML_TEXT);
             }
             // For END_ELEMENT we will return text, if any
             if (text != null) {
-                _mixedText = false;
+                _startElementAfterText = false;
                 _textValue = text;
                 return (_currentState = XML_TEXT);
             }
-            _mixedText = false;
+            _startElementAfterText = false;
             return _handleEndElement();
 
+        case XML_DELAYED_START_ELEMENT: // since 2.12, to support scalar Root Value
+            // Two cases: either "simple" with not text
+           if (_textValue == null) {
+               return _initStartElement();
+           }
+           // or one where there is first text (to translate into "":<text> key/value entry)
+           // then followed by start element
+           _startElementAfterText = true;
+           return (_currentState = XML_TEXT);
+
         case XML_ATTRIBUTE_NAME:
             // if we just returned name, will need to just send value next
             return (_currentState = XML_ATTRIBUTE_VALUE);
         case XML_TEXT:
             // mixed text with other elements
-            if (_mixedText) {
-                _mixedText = false;
+            if (_startElementAfterText) {
+                _startElementAfterText = false;
                 return _initStartElement();
             }
             // text followed by END_ELEMENT
             return _handleEndElement();
-
+        case XML_ROOT_TEXT:
+            close();
+            return (_currentState = XML_END);
         case XML_END:
             return XML_END;
 //            throw new IllegalStateException("No more XML tokens available (end of input)");
         }
         // Ok: must be END_ELEMENT; see what tag we get (or end)
-        switch (_skipUntilTag()) {
+        switch (_skipAndCollectTextUntilTag()) {
         case XMLStreamConstants.END_DOCUMENT:
+            close();
             return (_currentState = XML_END);
         case XMLStreamConstants.END_ELEMENT:
+            // 24-May-2020, tatu: Need to see if we have "mixed content" to offer
+            if (!_allWs(_textValue)) {
+                // _textValue already set
+                return (_currentState = XML_TEXT);
+            }
             return _handleEndElement();
         }
+        // 24-May-2020, tatu: Need to see if we have "mixed content" to offer
+        if (!_allWs(_textValue)) {
+            // _textValue already set
+            _startElementAfterText = true;
+            return (_currentState = XML_TEXT);
+        }
+
         // START_ELEMENT...
         return _initStartElement();
     }
 
+    /**
+     * @return Collected text, if any, EXCEPT that if {@code FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL}
+     *    AND empty element, returns {@code null}
+     */
     private final String _collectUntilTag() throws XMLStreamException
     {
         // 21-Jun-2017, tatu: Whether exposed as `null` or "" is now configurable...
@@ -438,18 +533,6 @@ public class XmlTokenStream
 
             case XMLStreamConstants.END_ELEMENT:
             case XMLStreamConstants.END_DOCUMENT:
-                // 04-May-2018, tatu: We could easily make <tag></tag> ALSO report
-                //    as `null`, by below, but that breaks existing tests so not
-                //    done at least until 3.0.
-                /*
-                if (chars == null) {
-                    if (FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL.enabledIn(_formatFeatures)) {
-                        return null;
-                    }
-                    return "";
-                }
-                return chars;
-                */
                 return (chars == null) ? "" : chars.toString();
 
             // note: SPACE is ignorable (and seldom seen), not to be included
@@ -474,6 +557,7 @@ public class XmlTokenStream
         }
     }
 
+    // Called to simply skip tokens until start/end tag, or end-of-document found
     private final int _skipUntilTag() throws XMLStreamException
     {
         while (_xmlReader.hasNext()) {
@@ -490,6 +574,45 @@ public class XmlTokenStream
         throw new IllegalStateException("Expected to find a tag, instead reached end of input");
     }
 
+    // Called to skip tokens until start/end tag (or end-of-document) found, but
+    // also collecting cdata until then, if any found, for possible "mixed content"
+    // to report
+    //
+    // @since 2.12
+    private final int _skipAndCollectTextUntilTag() throws XMLStreamException
+    {
+        CharSequence chars = null;
+
+        while (_xmlReader.hasNext()) {
+            int type;
+            switch (type = _xmlReader.next()) {
+            case XMLStreamConstants.START_ELEMENT:
+            case XMLStreamConstants.END_ELEMENT:
+            case XMLStreamConstants.END_DOCUMENT:
+                _textValue = (chars == null) ? "" : chars.toString();
+                return type;
+            // note: SPACE is ignorable (and seldom seen), not to be included
+            case XMLStreamConstants.CHARACTERS:
+            case XMLStreamConstants.CDATA:
+                {
+                    String str = _getText(_xmlReader);
+                    if (chars == null) {
+                        chars = str;
+                    } else  {
+                        if (chars instanceof String) {
+                            chars = new StringBuilder(chars);
+                        }
+                        ((StringBuilder)chars).append(str);
+                    }
+                }
+                break;
+            default:
+                // any other type (proc instr, comment etc) is just ignored
+            }
+        }
+        throw new IllegalStateException("Expected to find a tag, instead reached end of input");
+    }
+
     private final String _getText(XMLStreamReader2 r) throws XMLStreamException
     {
         try {
@@ -509,15 +632,6 @@ public class XmlTokenStream
     /**********************************************************************
      */
 
-    /*
-        _xmlReader = Stax2ReaderAdapter.wrapIfNecessary(xmlReader);
-        _currentState = XML_START_ELEMENT;
-        _localName = _xmlReader.getLocalName();
-        _namespaceURI = _xmlReader.getNamespaceURI();
-        _attributeCount = _xmlReader.getAttributeCount();
-        _formatFeatures = formatFeatures;
-     */
-    
     private final int _initStartElement() throws XMLStreamException
     {
         final String ns = _xmlReader.getNamespaceURI();
@@ -525,11 +639,10 @@ public class XmlTokenStream
 
         _checkXsiAttributes();
 
-        /* Support for virtual wrapping: in wrapping, may either
-         * create a new wrapper scope (if in sub-tree, or matches
-         * wrapper element itself), or implicitly close existing
-         * scope.
-         */
+        // Support for virtual wrapping: in wrapping, may either create a new
+        // wrapper scope (if in sub-tree, or matches wrapper element itself),
+        // or implicitly close existing scope.
+
         if (_currentWrapper != null) {
             if (_currentWrapper.matchesWrapper(localName, ns)) {
                 _currentWrapper = _currentWrapper.intermediateWrapper();
@@ -631,11 +744,21 @@ public class XmlTokenStream
 //System.out.println(" XMLTokenStream._handleEndElement(): IMPLICIT requestRepeat of END_ELEMENT '"+_localName);
             } else {
                 _currentWrapper = _currentWrapper.getParent();
+                // 23-May-2020, tatu: Let's clear _localName since it's value is unlikely
+                //    to be correct and we may or may not be able to get real one (for
+                //    END_ELEMENT could) -- FromXmlParser does NOT use this info
+                _localName = "";
+                _namespaceURI = "";
+
             }
+        } else {
+            // Not (necessarily) known, as per above, so:
+            _localName = "";
+            _namespaceURI = "";
         }
         return (_currentState = XML_END_ELEMENT);
     }
-    
+
     private JsonLocation _extractLocation(XMLStreamLocation2 location)
     {
         if (location == null) { // just for impls that might pass null...
@@ -647,8 +770,7 @@ public class XmlTokenStream
                 location.getColumnNumber());
     }
 
-
-    protected boolean _allWs(String str)
+    protected static boolean _allWs(String str)
     {
         final int len = (str == null) ? 0 : str.length();
         if (len > 0) {
@@ -661,7 +783,34 @@ public class XmlTokenStream
         return true;
     }
 
+    protected String _currentStateDesc() {
+        return _stateDesc(_currentState);
+    }
+
+    protected String _stateDesc(int state) {
+        switch (state) {
+        case XML_START_ELEMENT:
+            return "XML_START_ELEMENT";
+        case XML_END_ELEMENT:
+            return "XML_END_ELEMENT";
+        case XML_ATTRIBUTE_NAME:
+            return "XML_ATTRIBUTE_NAME";
+        case XML_ATTRIBUTE_VALUE:
+            return "XML_ATTRIBUTE_VALUE";
+        case XML_TEXT:
+            return "XML_TEXT";
+        case XML_DELAYED_START_ELEMENT:
+            return "XML_START_ELEMENT_DELAYED";
+        case XML_ROOT_TEXT:
+            return "XML_ROOT_TEXT";
+        case XML_END:
+            return "XML_END";
+        }
+        return "N/A ("+_currentState+")";
+    }
+
     // for DEBUGGING
+    /*
     @Override
     public String toString()
     {
@@ -670,4 +819,5 @@ public class XmlTokenStream
                 _currentState, _attributeCount, _nextAttributeIndex,
                 _localName, _textValue, _repeatElement, _currentWrapper, _repeatElement, _nextLocalName);
     }
+    */
 }
