@@ -5,6 +5,7 @@ import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.*;
 
+import javax.xml.XMLConstants;
 import javax.xml.namespace.QName;
 import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamWriter;
@@ -16,6 +17,7 @@ import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.base.GeneratorBase;
 import com.fasterxml.jackson.core.io.IOContext;
 import com.fasterxml.jackson.core.json.JsonWriteContext;
+import com.fasterxml.jackson.core.util.JacksonFeatureSet;
 import com.fasterxml.jackson.dataformat.xml.XmlPrettyPrinter;
 import com.fasterxml.jackson.dataformat.xml.util.DefaultXmlPrettyPrinter;
 import com.fasterxml.jackson.dataformat.xml.util.StaxUtil;
@@ -23,11 +25,12 @@ import com.fasterxml.jackson.dataformat.xml.util.StaxUtil;
 /**
  * {@link JsonGenerator} that outputs JAXB-style XML output instead of JSON content.
  * Operation requires calling code (usually either standard Jackson serializers,
- * or in some cases (like <code>BeanSerializer</code>) customised ones) to do
+ * or in some cases (like <code>BeanSerializer</code>) customized ones) to do
  * additional configuration calls beyond regular {@link JsonGenerator} API,
  * mostly to pass namespace information.
  */
-public final class ToXmlGenerator
+public class ToXmlGenerator
+// non-final since 2.12 but only sub-class if you really know what you are doing...
     extends GeneratorBase
 {
     /**
@@ -35,7 +38,7 @@ public final class ToXmlGenerator
      * name to use...
      */
     protected final static String DEFAULT_UNKNOWN_ELEMENT = "unknown";
-    
+
     /**
      * Enumeration that defines all togglable extra XML-specific features
      */
@@ -310,6 +313,11 @@ public final class ToXmlGenerator
     @Override
     public boolean canWriteFormattedNumbers() { return true; }
 
+    @Override // @since 2.12
+    public JacksonFeatureSet<StreamWriteCapability> getWriteCapabilities() {
+        return DEFAULT_TEXTUAL_WRITE_CAPABILITIES;
+    }
+
     // @since 2.7.5
     public boolean inRoot() {
         return _writeContext.inRoot();
@@ -834,22 +842,24 @@ public final class ToXmlGenerator
         if (_nextName == null) {
             handleMissingName();
         }
+        final org.codehaus.stax2.typed.Base64Variant stax2base64v = StaxUtil.toStax2Base64Variant(b64variant);
         try {
             if (_nextIsAttribute) {
                 // Stax2 API only has 'full buffer' write method:
                 byte[] fullBuffer = toFullBuffer(data, offset, len);
-                _xmlWriter.writeBinaryAttribute("", _nextName.getNamespaceURI(), _nextName.getLocalPart(), fullBuffer);
+                _xmlWriter.writeBinaryAttribute(stax2base64v,
+                        "", _nextName.getNamespaceURI(), _nextName.getLocalPart(), fullBuffer);
             } else if (checkNextIsUnwrapped()) {
             	// should we consider pretty-printing or not?
-                _xmlWriter.writeBinary(data, offset, len);
+                _xmlWriter.writeBinary(stax2base64v, data, offset, len);
             } else {
                 if (_xmlPrettyPrinter != null) {
                     _xmlPrettyPrinter.writeLeafElement(_xmlWriter,
                             _nextName.getNamespaceURI(), _nextName.getLocalPart(),
-                            data, offset, len);
+                            stax2base64v, data, offset, len);
                 } else {
                     _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
-                    _xmlWriter.writeBinary(data, offset, len);
+                    _xmlWriter.writeBinary(stax2base64v, data, offset, len);
                     _xmlWriter.writeEndElement();
                 }
             }
@@ -869,23 +879,25 @@ public final class ToXmlGenerator
         if (_nextName == null) {
             handleMissingName();
         }
+        final org.codehaus.stax2.typed.Base64Variant stax2base64v = StaxUtil.toStax2Base64Variant(b64variant);
         try {
             if (_nextIsAttribute) {
                 // Stax2 API only has 'full buffer' write method:
                 byte[] fullBuffer = toFullBuffer(data, dataLength);
-                _xmlWriter.writeBinaryAttribute("", _nextName.getNamespaceURI(), _nextName.getLocalPart(), fullBuffer);
+                _xmlWriter.writeBinaryAttribute(stax2base64v,
+                        "", _nextName.getNamespaceURI(), _nextName.getLocalPart(), fullBuffer);
             } else if (checkNextIsUnwrapped()) {
               // should we consider pretty-printing or not?
-                writeStreamAsBinary(data, dataLength);
+                writeStreamAsBinary(stax2base64v, data, dataLength);
 
             } else {
                 if (_xmlPrettyPrinter != null) {
                     _xmlPrettyPrinter.writeLeafElement(_xmlWriter,
                             _nextName.getNamespaceURI(), _nextName.getLocalPart(),
-                            toFullBuffer(data, dataLength), 0, dataLength);
+                            stax2base64v, toFullBuffer(data, dataLength), 0, dataLength);
                 } else {
                     _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
-                    writeStreamAsBinary(data, dataLength);
+                    writeStreamAsBinary(stax2base64v, data, dataLength);
                     _xmlWriter.writeEndElement();
                 }
             }
@@ -896,7 +908,8 @@ public final class ToXmlGenerator
         return dataLength;
     }
 
-    private void writeStreamAsBinary(InputStream data, int len) throws IOException, XMLStreamException 
+    private void writeStreamAsBinary(org.codehaus.stax2.typed.Base64Variant stax2base64v,
+            InputStream data, int len) throws IOException, XMLStreamException 
     {
         // base64 encodes up to 3 bytes into a 4 bytes string
         byte[] tmp = new byte[3];
@@ -907,7 +920,7 @@ public final class ToXmlGenerator
             len -= read;
             if(offset == 3) {
                 offset = 0;
-                _xmlWriter.writeBinary(tmp, 0, 3);
+                _xmlWriter.writeBinary(stax2base64v, tmp, 0, 3);
             }
             if (len == 0) {
                 break;
@@ -916,11 +929,10 @@ public final class ToXmlGenerator
 
         // we still have < 3 bytes in the buffer
         if(offset > 0) {
-            _xmlWriter.writeBinary(tmp, 0, offset);
+            _xmlWriter.writeBinary(stax2base64v, tmp, 0, offset);
         }
     }
 
-    
     private byte[] toFullBuffer(byte[] data, int offset, int len)
     {
         // might already be ok:
@@ -991,20 +1003,32 @@ public final class ToXmlGenerator
         if (_nextName == null) {
             handleMissingName();
         }
-        // !!! TODO: proper use of 'xsd:isNil' ?
         try {
             if (_nextIsAttribute) {
-                /* With attributes, best just leave it out, right? (since there's no way
-                 * to use 'xsi:nil')
-                 */
+                // With attributes, best just leave it out, right? (since there's no way
+                // to use 'xsi:nil')
             } else if (checkNextIsUnwrapped()) {
             	// as with above, best left unwritten?
             } else {
+                final boolean asXsiNil = isEnabled(Feature.WRITE_NULLS_AS_XSI_NIL);
                 if (_xmlPrettyPrinter != null) {
-                	_xmlPrettyPrinter.writeLeafNullElement(_xmlWriter,
-                			_nextName.getNamespaceURI(), _nextName.getLocalPart());
+                    // 12-Nov-2020, tatu: Not clean, due to backwards-compat challenges..
+                    //    but has to do
+                    if (asXsiNil && (_xmlPrettyPrinter instanceof DefaultXmlPrettyPrinter)) {
+                        ((DefaultXmlPrettyPrinter) _xmlPrettyPrinter).writeLeafXsiNilElement(_xmlWriter,
+                                _nextName.getNamespaceURI(), _nextName.getLocalPart());
+                    } else {
+                        _xmlPrettyPrinter.writeLeafNullElement(_xmlWriter,
+                                _nextName.getNamespaceURI(), _nextName.getLocalPart());
+                    }
                 } else {
-	            	_xmlWriter.writeEmptyElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
+                    if (asXsiNil) {
+                        _xmlWriter.writeStartElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
+                        _xmlWriter.writeAttribute("xsi", XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI, "nil", "true");
+                        _xmlWriter.writeEndElement();
+                    } else {
+                        _xmlWriter.writeEmptyElement(_nextName.getNamespaceURI(), _nextName.getLocalPart());
+                    }
                 }
             }
         } catch (XMLStreamException e) {
