@@ -12,6 +12,7 @@ import com.fasterxml.jackson.databind.type.TypeFactory;
 import com.fasterxml.jackson.dataformat.xml.XmlAnnotationIntrospector;
 import com.fasterxml.jackson.dataformat.xml.XmlMapper;
 import com.fasterxml.jackson.dataformat.xml.XmlTestBase;
+import com.fasterxml.jackson.module.jaxb.JaxbAnnotationIntrospector;
 
 // Test for [dataformat-xml#291]: works via field, not constructor
 // (name mismatch to fix in test case)
@@ -27,7 +28,7 @@ public class BuilderWithJAXB291Test extends XmlTestBase
         private final String country;
         private final String county;
 
-        private Address(AddressBuilder addressbuilder) {
+        Address(AddressBuilder addressbuilder) {
             this.address1 = addressbuilder.address1;
             this.city = addressbuilder.city;
             this.stateProvince = addressbuilder.stateProvince;
@@ -118,7 +119,7 @@ public class BuilderWithJAXB291Test extends XmlTestBase
                 "    <PostalCode>94132</PostalCode>\n" + 
                 "</Address>";
 
-        XmlJaxbAnnotationIntrospector xmlIntr = new XmlJaxbAnnotationIntrospector(TypeFactory.defaultInstance());
+        AnnotationIntrospector xmlIntr = new JaxbAnnotationIntrospector(TypeFactory.defaultInstance());
         AnnotationIntrospector intr = XmlAnnotationIntrospector.Pair.instance
                 (xmlIntr, new JacksonAnnotationIntrospector());
         XmlMapper mapper = mapperBuilder()
