@@ -3,7 +3,7 @@ package com.fasterxml.jackson.dataformat.xml.stream;
 import java.io.*;
 
 import com.fasterxml.jackson.core.*;
-
+import com.fasterxml.jackson.core.JsonParser.NumberType;
 import com.fasterxml.jackson.databind.JsonNode;
 import com.fasterxml.jackson.databind.ObjectMapper;
 
@@ -29,29 +29,103 @@ public class XmlParserTest extends XmlTestBase
 
     /*
     /**********************************************************
-    /* Unit tests
+    /* Unit tests, simplest/manual
     /**********************************************************
      */
     
     public void testSimplest() throws Exception
     {
-        assertEquals("{\"leaf\":\"abc\"}",
-                _readXmlWriteJson("<root><leaf>abc</leaf></root>"));
+        final String XML = "<root><leaf>abc</leaf></root>";
+        // -> "{\"leaf\":\"abc\"}"
+
+        try (JsonParser p = _xmlMapper.createParser(XML)) {
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("leaf", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("abc", p.getText());
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+        }
     }
 
     public void testSimpleWithEmpty() throws Exception
     {
         // 21-Jun-2017, tatu: Depends on setting actually...
-        XmlFactory f = new XmlFactory();
 
-        f.enable(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL);
-        assertEquals("{\"leaf\":null}",
-                _readXmlWriteJson(f, "<root><leaf /></root>"));
-        f.disable(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL);
-        assertEquals("{\"leaf\":\"\"}",
-                _readXmlWriteJson(f, "<root><leaf /></root>"));
+        final String XML = "<root><leaf /></root>";
+
+        // -> "{"leaf":null}"
+        try (JsonParser p = _xmlMapper.reader().with(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL)
+                .createParser(XML)) {
+            assertTrue(((FromXmlParser) p).isEnabled(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL));
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("leaf", p.currentName());
+            assertToken(JsonToken.VALUE_NULL, p.nextToken());
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+        }
+
+        // -> "{"leaf":""}"
+        try (JsonParser p = _xmlMapper.reader().without(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL)
+                .createParser(XML)) {
+            assertFalse(((FromXmlParser) p).isEnabled(FromXmlParser.Feature.EMPTY_ELEMENT_AS_NULL));
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("leaf", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("", p.getText());
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+        }
     }
 
+    /**
+     * Test that verifies coercion of a "simple" cdata segment within root element
+     * as matching scalar token, similar to how other elements work.
+     */
+    public void testRootScalar() throws Exception
+    {
+        // 02-Jul-2020, tatu: Does not work quite yet
+        final String XML = "<data>value</data>";
+        try (JsonParser p = _xmlMapper.createParser(XML)) {
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("value", p.getText());
+            assertNull(p.nextToken());
+            // should be ok to call again tho
+            assertNull(p.nextToken());
+        }
+    }
+
+    public void testRootMixed() throws Exception
+    {
+        // 02-Jul-2020, tatu: Does not work quite yet
+        final String XML = "<data>value<child>abc</child></data>";
+        try (JsonParser p = _xmlMapper.createParser(XML)) {
+            assertToken(JsonToken.START_OBJECT, p.nextToken());
+
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("value", p.getText());
+
+            assertToken(JsonToken.FIELD_NAME, p.nextToken());
+            assertEquals("child", p.currentName());
+            assertToken(JsonToken.VALUE_STRING, p.nextToken());
+            assertEquals("abc", p.getText());
+
+            assertToken(JsonToken.END_OBJECT, p.nextToken());
+            assertNull(p.nextToken());
+        }
+    }
+    
+    /*
+    /**********************************************************
+    /* Unit tests, slightly bigger, automated
+    /**********************************************************
+     */
+    
     public void testSimpleNested() throws Exception
     {
         assertEquals("{\"a\":{\"b\":{\"c\":\"xyz\"}}}",
@@ -76,7 +150,7 @@ public class XmlParserTest extends XmlTestBase
         // Former could be worked around; latter less so at this point.
 
         // So, for now, let's just do sort of minimal verification, manually
-        JsonParser p = _xmlMapper.getFactory().createParser(xml);
+        JsonParser p = _xmlMapper.createParser(xml);
         
         assertToken(JsonToken.START_OBJECT, p.nextToken()); // main object
 
@@ -181,7 +255,7 @@ public class XmlParserTest extends XmlTestBase
         xp.close();
 
         // And then with array handling:
-        xp = (FromXmlParser) _xmlFactory.createParser(new StringReader(XML));
+        xp = (FromXmlParser) _xmlMapper.createParser(XML);
         assertTrue(xp.getParsingContext().inRoot());
 
         assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <array>
@@ -224,7 +298,7 @@ public class XmlParserTest extends XmlTestBase
     {
         final String XML = "<data max=\"7\" offset=\"9\"/>";
 
-        FromXmlParser xp = (FromXmlParser) _xmlFactory.createParser(new StringReader(XML));
+        FromXmlParser xp = (FromXmlParser) _xmlMapper.createParser(XML);
 
         // First: verify handling without forcing array handling:
         assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <data>
@@ -251,6 +325,50 @@ public class XmlParserTest extends XmlTestBase
         xp.close();
     }
 
+    public void testMixedContent() throws Exception
+    {
+        String exp = a2q("{'':'first','a':'123','':'second','b':'456','':'last'}");
+        String result = _readXmlWriteJson("<root>first<a>123</a>second<b>456</b>last</root>");
+
+//System.err.println("result = \n"+result);
+        assertEquals(exp, result);
+    }
+
+    public void testInferredNumbers() throws Exception
+    {
+        final String XML = "<data value1='abc' value2='42'>123456789012</data>";
+
+        FromXmlParser xp = (FromXmlParser) _xmlMapper.createParser(XML);
+
+        // First: verify handling without forcing array handling:
+        assertToken(JsonToken.START_OBJECT, xp.nextToken()); // <data>
+        assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // @value1
+        assertEquals("value1", xp.currentName());
+        assertToken(JsonToken.VALUE_STRING, xp.nextToken());
+        assertFalse(xp.isExpectedNumberIntToken());
+        assertEquals("abc", xp.getText());
+
+        assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // @value2
+        assertEquals("value2", xp.getCurrentName());
+        assertToken(JsonToken.VALUE_STRING, xp.nextToken());
+        assertTrue(xp.isExpectedNumberIntToken());
+        assertEquals(JsonToken.VALUE_NUMBER_INT, xp.currentToken());
+        assertEquals(NumberType.INT, xp.getNumberType());
+        assertEquals(42, xp.getIntValue());
+
+        assertToken(JsonToken.FIELD_NAME, xp.nextToken()); // implicit for text
+        assertEquals("", xp.getCurrentName());
+
+        assertToken(JsonToken.VALUE_STRING, xp.nextToken());
+        assertTrue(xp.isExpectedNumberIntToken());
+        assertEquals(JsonToken.VALUE_NUMBER_INT, xp.currentToken());
+        assertEquals(NumberType.LONG, xp.getNumberType());
+        assertEquals(123456789012L, xp.getLongValue());
+        
+        assertToken(JsonToken.END_OBJECT, xp.nextToken()); // </data>
+        xp.close();
+    }
+
     /*
     /**********************************************************
     /* Helper methods
